.include "m8def.inc"				; Deklarationen fr ATmega8

.equ	timeout = 10000				; Eingabetimeout in 1/10ms
.equ	softdiv = 100				; Softwareteiler
.equ	startbpm = 120				; BPM Startwert

; Register Definitionen

.def	timeL = R0					; Zeitspeicher
.def	timeH = R1

.def	compL = R2					; Zeitrichtwert in 1/10ms
.def	compH = R3

.def	cnt1L = R26					; Zhler fr Beat-Tap
.def	cnt1H = R27

.def	cnt2L = R28					; Zhler fr Lauflicht
.def	cnt2H = R29

.def	div = R4					; Divisorspeicher
.def	stat = R25					; Statusregister

; Statusregister stat
.equ	cnt = 7						; Beat-Tap-Counter cnt1 luft
.equ	ovf = 6						; Overflow in Zeitspeicherregister time
.equ	hold = 5					; Manual Beat-Tap


.cseg								; Programm-Flash
		rjmp	main				; Reset-Einsprung

.org	INT0addr					; 0x001 External Interrupt Request 0 @ PD2
		rjmp	inta				; Springe zu inta

.org	INT1addr					; 0x002 External Interrupt Request 1 @ PD3
		rjmp	intb				; Springe zu inta

.org	OC1Aaddr					; 0x006 Timer/Counter1 Compare Match A
		rjmp	timer				; Springe zu timer

.org	0x013						; Interrupteinsprnge bergehen



/************************************************************************/
/*							Initialisierung								*/
/************************************************************************/

main:	ldi		R16, LOW(RAMEND)	; Stapel anlegen
		out		SPL, R16
		ldi		R16, HIGH(RAMEND)
		out		SPH, R16

		; INT0 und INT1 Interrupt bei fallender Flanke
        ldi		R16, (1 << ISC01) | (1 << ISC11)
		out		MCUCR, R16

		; INT0 und INT1 Interrupt aktivieren
		ldi		R16, (1 << INT0) | (1 << INT1)
		out		GICR, R16

		; Timer1 mit Vorteiler 8 starten
		; Leeren nach Compare Match
		ldi		R16, (1 << WGM12) | (1 << CS11)
		out		TCCR1B, R16

		ldi		R16, (1 << OCIE1A)	; Timer1 Compare Match A Interrupt aktivieren
		out		TIMSK, R16

		ldi		R16, (softdiv-1)	; Comparewert Softwareteiler laden
		out		OCR1AL, R16

		sei							; Interrupts aktivieren

		; BPM Startwert laden
		ldi		R16, LOW(600000/startbpm)
		mov		compL, R16
		ldi		R16, HIGH(600000/startbpm)
		mov		compH, R16

		ldi		R16, 0b00000011		; PD0 und PD1 als Ausgang definieren
		out		DDRD, R16

		ser		R16					; Port B als Ausgang definieren
		out		DDRB, R16

		ser		R16					; Port C als Ausgang definieren
		out		DDRC, R16

		; Zu benutzende Register leeren
		clr		timeL
		clr		timeH

		clr		cnt1L
		clr		cnt1H

		clr		cnt2L
		clr		cnt2H

		clr		div	
		clr		stat

/************************************************************************/
/*							Endlosschleife								*/
/************************************************************************/

loop:	rjmp	loop;				; Endlosschleife



/************************************************************************/
/*							INT0 Interrupt								*/
/************************************************************************/


inta:	clr		cnt2L				; Lauflichtzhler zurcksetzen
		clr		cnt2H

		rcall	lights

		sbrc	stat, hold			; Kehre zurck wenn hold=1
		reti

		sbrc	stat, cnt			; Springe zu inta1 wenn cnt=1
		rjmp	inta1

		clr		timeL				; Zeitspeicher zurcksetzen
		clr		timeH

		clr		div					; Divisorspeicher zurcksetzen
		out		PORTC, div			; Divisor an Port C ausgeben

		cbr		stat, (1 << ovf)	; Setze ovf im Statusregister stat auf 0
		sbr		stat, (1 << cnt)	; Setze cnt im Statusregister stat auf 1

		sbi		PORTD, 0			; Status-LED Grn
		cbi		PORTD, 1

		reti						; Kehre zurck

inta1:	sbrs	stat, ovf			; Springe zu inta2 wenn ovf=0
		rjmp	inta2

		clr		cnt1L				; Beat-Tap Zhler zurcksetzen
		clr		cnt1H

		reti						; Kehre zurck

inta2:	add		timeL, cnt1L		; Addiere Beat-Tap Zhler zu Zeitspeicher
		adc		timeH, cnt1H

		brcc	inta3				; Springe zu inta2 wenn kein Overflow

		clr		cnt1L				; Beat-Tap Zhler zurcksetzen
		clr		cnt1H

		sbr		stat, (1 << ovf)	; Overflow Bit im Statusregister stat setzen

		cbi		PORTD, 0			; Status-LED Rot
		sbi		PORTD, 1

		reti						; Kehre zurck

inta3:	clr		cnt1L				; Beat-Tap Zhler zurcksetzen
		clr		cnt1H

		inc		div					; Divisor erhhen

		out		PORTC, div			; Divisor an Port C ausgeben

		; Divisionsroutine			; Zeitrichtwert = Zeitspeicher / Divisor
		; R16 = Hilfsregister
		; timeL = Divident LOW
		; timeH = Divident HIGH
		; div = Divisor
		; compL = Ergebnis LOW
		; compH = Ergebnis HIGH

		clr		R16					; Leere Hilfsregister R16
		clr		compL				; Leere Divisionsergebnisregister
		clr		compH
		inc		compL				; Belege Ergebnisregister mit 1 vor

		push	timeL				; Zeitspeicherregister retten
		push	timeH

divi:	clc							; Carry-Bit lschen

		rol		timeL				; Nchsthheres Bit des Dividenden in das Hilfsregister rotieren
		rol		timeH
		rol		R16					; Hilfsregister mit 2 Multiplizieren

		brcs	divi1				; Eine 1 ist herausgerollt, subtrahiere
		cp		R16, div			; Divisionsergebnis 1 oder 0?

		brcs	divi2				; berspringe Subtraktion, wenn kleiner

divi1:	sub		R16, div			; Subtrahiere Divisor
		sec							; Setze Carry-Bit
		rjmp	divi3				; Zum Schieben des Ergebnisses

divi2:	clc							; Lsche Carry-Bit

divi3:	rol		compL				; Rotiere Carry-Bit in das Ergebnis
		rol		compH
		brcc	divi

		pop		timeH				; Zeitspeicherregister wiederherstellen
		pop		timeL

		reti						; Kehre zurck

/************************************************************************/
/*							INT2 Interrupt								*/
/************************************************************************/

intb:	ldi		R16, (1 << hold)	; Maske fr hold-bit
		eor		stat, R16			; Hold komplementieren

		clr		cnt1L				; Beat-Tap Zhler zurcksetzen
		clr		cnt1H

		reti						; Kehre zurck



/************************************************************************/
/*					Timer2 Compare Match Interrupt						*/
/************************************************************************/

timer:	adiw	cnt2L, 1			; Lauflichtzhler erhhen

		cp		cnt2L, compL		; Vergleiche Lauflichtzhler mit Zeitrichtwert
		cpc		cnt2H, compH

		brne	timer1				; Springe zu timer1 wenn ungleich

		clr		cnt2L				; Lauflichtzhler zurcksetzen
		clr		cnt2H

		sbrs	stat, hold			; Rufe lights auf wenn hold=0
		rcall	lights

timer1:	sbrs	stat, cnt			; Kehre zurck wenn cnt=0
		reti

		adiw	cnt1L, 1			; Beat-Tap Zhler erhhen

		ldi		R16, LOW(timeout)	; Lade Timeout in die Arbeitsregister R16/R17
		ldi		R17, HIGH(timeout)

		cp		cnt1L, R16			; Vergleiche Beat-Tap Zhler mit 10000
		cpc		cnt1H, R17

		breq	timer2				; Kehre zurck wenn ungleich
		reti

timer2:	clr		cnt1L				; Beat-Tap Zhler zurcksetzen
		clr		cnt1H

		; Setzte ovr und cnt im Statusregister stat auf 0
		cbr		stat, (1 << ovf) | (1 << cnt)

		cbi		PORTD, 0			; Status-LED aus
		cbi		PORTD, 1

		reti						; Kehre zurck



/************************************************************************/
/*							Lauflichtaktion								*/
/************************************************************************/

lights: in		R16, PORTB			; Port B in R16 laden
		com		R16					; R16 komplementieren
		out		PORTB, R16			; R16 an Port B ausgeben
		ret
